=== modified file 'drizzled/plugin/authorization.cc'
--- drizzled/plugin/authorization.cc	2010-03-31 05:53:34 +0000
+++ drizzled/plugin/authorization.cc	2010-04-15 19:13:57 +0000
@@ -24,6 +24,7 @@
 
 #include "drizzled/plugin/authorization.h"
 #include "drizzled/security_context.h"
+#include "drizzled/table_identifier.h"
 #include "drizzled/error.h"
 #include "drizzled/session.h"
 #include "drizzled/plugin/registry.h"
@@ -61,10 +62,10 @@
   public unary_function<plugin::Authorization *, bool>
 {
   const SecurityContext &user_ctx;
-  const string &schema;
+  SchemaIdentifier &schema;
 public:
   RestrictDbFunctor(const SecurityContext &user_ctx_arg,
-                    const string &schema_arg) :
+                    SchemaIdentifier &schema_arg) :
     unary_function<plugin::Authorization *, bool>(),
     user_ctx(user_ctx_arg),
     schema(schema_arg)
@@ -80,21 +81,18 @@
   public unary_function<plugin::Authorization *, bool>
 {
   const SecurityContext &user_ctx;
-  const string &schema;
-  const string &table;
+  TableIdentifier &table;
 public:
   RestrictTableFunctor(const SecurityContext &user_ctx_arg,
-                       const string &schema_arg,
-                       const string &table_arg) :
+                       TableIdentifier &table_arg) :
     unary_function<plugin::Authorization *, bool>(),
     user_ctx(user_ctx_arg),
-    schema(schema_arg),
     table(table_arg)
   { }
 
   inline result_type operator()(argument_type auth)
   {
-    return auth->restrictTable(user_ctx, schema, table);
+    return auth->restrictTable(user_ctx, table);
   }
 };
 
@@ -117,6 +115,22 @@
   }
 };
 
+class PruneSchemaFunctor :
+  public unary_function<SchemaIdentifier&, bool>
+{
+  const SecurityContext &user_ctx;
+public:
+  PruneSchemaFunctor(const SecurityContext &user_ctx_arg) :
+    unary_function<SchemaIdentifier&, bool>(),
+    user_ctx(user_ctx_arg)
+  { }
+
+  inline result_type operator()(argument_type auth)
+  {
+    return not plugin::Authorization::isAuthorized(user_ctx, auth, false);
+  }
+};
+
 } /* namespace */
 
 bool plugin::Authorization::isAuthorized(const SecurityContext &user_ctx,
@@ -131,7 +145,8 @@
   vector<plugin::Authorization *>::const_iterator iter=
     find_if(authorization_plugins.begin(),
             authorization_plugins.end(),
-            RestrictDbFunctor(user_ctx, schema_identifier.getPath()));
+            RestrictDbFunctor(user_ctx, schema_identifier));
+
 
   /*
    * If iter is == end() here, that means that all of the plugins returned
@@ -153,8 +168,7 @@
 }
 
 bool plugin::Authorization::isAuthorized(const SecurityContext &user_ctx,
-                                         const string &schema,
-                                         const string &table,
+                                         TableIdentifier &table,
                                          bool send_error)
 {
   /* If we never loaded any authorization plugins, just return true */
@@ -165,7 +179,7 @@
   vector<plugin::Authorization *>::const_iterator iter=
     find_if(authorization_plugins.begin(),
             authorization_plugins.end(),
-            RestrictTableFunctor(user_ctx, schema, table));
+            RestrictTableFunctor(user_ctx, table));
 
   /*
    * If iter is == end() here, that means that all of the plugins returned
@@ -179,7 +193,7 @@
       my_error(ER_DBACCESS_DENIED_ERROR, MYF(0),
                user_ctx.getUser().c_str(),
                user_ctx.getIp().c_str(),
-               schema.c_str());
+               table.getSQLPath().c_str());
     }
     return false;
   }
@@ -222,24 +236,14 @@
 void plugin::Authorization::pruneSchemaNames(const SecurityContext &user_ctx,
                                              SchemaIdentifierList &set_of_schemas)
 {
-  SchemaIdentifierList pruned_set_of_names;
-
   /* If we never loaded any authorization plugins, just return true */
   if (authorization_plugins.empty())
     return;
 
-  /**
-   * @TODO: It would be stellar if we could find a way to do this with a
-   * functor and an STL algoritm
-   */
-  for (SchemaIdentifierList::iterator iter; iter != set_of_schemas.end(); iter++)
-  {
-    if (not plugin::Authorization::isAuthorized(user_ctx, *iter, false))
-    {
-      iter= pruned_set_of_names.erase(iter);
-    }
-  }
-  set_of_schemas.swap(pruned_set_of_names);
+  set_of_schemas.erase(remove_if(set_of_schemas.begin(),
+                                 set_of_schemas.end(),
+                                 PruneSchemaFunctor(user_ctx)),
+                       set_of_schemas.end());
 }
 
 } /* namespace drizzled */

=== modified file 'drizzled/plugin/authorization.h'
--- drizzled/plugin/authorization.h	2010-03-31 05:53:34 +0000
+++ drizzled/plugin/authorization.h	2010-04-15 19:13:57 +0000
@@ -56,7 +56,7 @@
    * @returns true if the user cannot access the schema
    */
   virtual bool restrictSchema(const SecurityContext &user_ctx,
-                              const std::string &schema)= 0;
+                              SchemaIdentifier &schema)= 0;
 
   /**
    * Should we restrict the current user's access to this table?
@@ -68,8 +68,7 @@
    * @returns true if the user cannot access the table
    */
   virtual bool restrictTable(const SecurityContext &user_ctx,
-                             const std::string &schema,
-                             const std::string &table);
+                             TableIdentifier &table);
 
   /**
    * Should we restrict the current user's access to see this process?
@@ -90,8 +89,7 @@
 
   /** Server API method for checking table authorization */
   static bool isAuthorized(const SecurityContext &user_ctx,
-                           const std::string &schema,
-                           const std::string &table,
+                           TableIdentifier &table_identifier,
                            bool send_error= true);
 
   /** Server API method for checking process authorization */
@@ -115,10 +113,9 @@
 };
 
 inline bool Authorization::restrictTable(const SecurityContext &user_ctx,
-                                         const std::string &schema,
-                                         const std::string &)
+                                         TableIdentifier &table)
 {
-  return restrictSchema(user_ctx, schema);
+  return restrictSchema(user_ctx, table);
 }
 
 inline bool Authorization::restrictProcess(const SecurityContext &,

=== modified file 'drizzled/sql_base.cc'
--- drizzled/sql_base.cc	2010-04-01 15:54:57 +0000
+++ drizzled/sql_base.cc	2010-04-15 19:13:57 +0000
@@ -2102,9 +2102,9 @@
      * to see if it exists so that an unauthorized user cannot phish for
      * table/schema information via error messages
      */
+    TableIdentifier the_table(tables->db, tables->table_name);
     if (not plugin::Authorization::isAuthorized(getSecurityContext(),
-                                                string(tables->db),
-                                                string(tables->table_name)))
+                                                TableIdentifier(tables->db, tables->table_name)))
     {
       result= -1;                               // Fatal error
       break;

